"""Tests for test_pioneer.executor.run.executor_run and executor_run_folder"""
import json
from unittest.mock import patch, MagicMock

from test_pioneer.executor.run.executor_run import run
from test_pioneer.executor.run.executor_run_folder import run_folder


class TestRun:
    @patch("test_pioneer.executor.run.executor_run.select_with_runner")
    def test_runner_selection_failure(self, mock_select):
        mock_select.return_value = (False, None)
        result = run({"run": "test.json", "with": "api-runner"})
        assert result is False

    @patch("test_pioneer.executor.run.executor_run.select_with_runner")
    def test_non_string_path_returns_false(self, mock_select):
        mock_select.return_value = (True, MagicMock())
        result = run({"run": 123, "with": "api-runner"})
        assert result is False

    @patch("test_pioneer.executor.run.executor_run.select_with_runner")
    def test_file_not_exist_returns_false(self, mock_select):
        mock_select.return_value = (True, MagicMock())
        result = run({"run": "nonexistent_file.json", "with": "api-runner"})
        assert result is False

    @patch("test_pioneer.executor.run.executor_run.select_with_runner")
    def test_valid_json_file(self, mock_select, tmp_path):
        mock_runner = MagicMock()
        mock_select.return_value = (True, mock_runner)

        json_file = tmp_path / "test.json"
        json_file.write_text(json.dumps({"action": "test"}), encoding="utf-8")

        with patch("os.getcwd", return_value=str(tmp_path)):
            result = run({"run": "test.json", "with": "api-runner"})
        assert result is True
        mock_runner.assert_called_once_with({"action": "test"})

    @patch("test_pioneer.executor.run.executor_run.select_with_runner")
    def test_invalid_json_returns_false(self, mock_select, tmp_path):
        mock_select.return_value = (True, MagicMock())

        json_file = tmp_path / "bad.json"
        json_file.write_text("not json content", encoding="utf-8")

        with patch("os.getcwd", return_value=str(tmp_path)):
            result = run({"run": "bad.json", "with": "api-runner"})
        assert result is False


class TestRunFolder:
    @patch("test_pioneer.executor.run.executor_run_folder.select_with_runner")
    def test_runner_selection_failure(self, mock_select):
        mock_select.return_value = (False, None)
        result = run_folder({"run_folder": "./test", "with": "api-runner"})
        assert result is False

    @patch("test_pioneer.executor.run.executor_run_folder.select_with_runner")
    def test_non_string_path_returns_false(self, mock_select):
        mock_select.return_value = (True, MagicMock())
        result = run_folder({"run_folder": 123, "with": "api-runner"})
        assert result is False

    @patch("test_pioneer.executor.run.executor_run_folder.select_with_runner")
    def test_folder_not_exist_returns_false(self, mock_select):
        mock_select.return_value = (True, MagicMock())
        result = run_folder({"run_folder": "nonexistent_folder", "with": "api-runner"})
        assert result is False

    @patch("test_pioneer.executor.run.executor_run_folder.select_with_runner")
    def test_empty_folder_returns_false(self, mock_select, tmp_path):
        mock_select.return_value = (True, MagicMock())
        empty_dir = tmp_path / "empty"
        empty_dir.mkdir()

        with patch("os.getcwd", return_value=str(tmp_path)):
            result = run_folder({"run_folder": "empty", "with": "api-runner"})
        assert result is False

    @patch("test_pioneer.executor.run.executor_run_folder.select_with_runner")
    def test_folder_with_json_files(self, mock_select, tmp_path):
        mock_runner = MagicMock()
        mock_select.return_value = (True, mock_runner)

        test_dir = tmp_path / "tests"
        test_dir.mkdir()
        (test_dir / "test1.json").write_text("{}", encoding="utf-8")
        (test_dir / "test2.json").write_text("{}", encoding="utf-8")

        with patch("os.getcwd", return_value=str(tmp_path)):
            result = run_folder({"run_folder": "tests", "with": "gui-runner"})
        assert result is True
        mock_runner.assert_called_once()
        # Verify JSON files were passed
        json_files = mock_runner.call_args[0][0]
        assert len(json_files) == 2



"""Tests for test_pioneer.executor.run.parallel_run"""
from unittest.mock import patch, MagicMock

from test_pioneer.executor.run.parallel_run import parallel_run


class TestParallelRun:
    def test_missing_parallel_run_tag(self):
        result = parallel_run({"parallel_run": None})
        assert result is False

    def test_mismatched_runners_and_scripts(self):
        step = {
            "parallel_run": {
                "runners": ["api-runner", "web-runner"],
                "scripts": ["test1.json"],
            }
        }
        result = parallel_run(step)
        assert result is False

    @patch("test_pioneer.executor.run.parallel_run.is_installed", return_value=False)
    def test_gui_runner_not_installed(self, mock_installed):
        step = {
            "parallel_run": {
                "runners": ["gui-runner"],
                "scripts": ["test.json"],
            }
        }
        result = parallel_run(step)
        assert result is False

    @patch("test_pioneer.executor.run.parallel_run.subprocess.Popen")
    @patch("test_pioneer.executor.run.parallel_run.is_installed", return_value=False)
    def test_script_not_found_skips(self, mock_installed, mock_popen, tmp_path):
        step = {
            "parallel_run": {
                "runners": ["api-runner"],
                "scripts": [str(tmp_path / "nonexistent.json")],
            }
        }
        result = parallel_run(step)
        assert result is True
        mock_popen.assert_not_called()

    @patch("test_pioneer.executor.run.parallel_run.subprocess.Popen")
    @patch("test_pioneer.executor.run.parallel_run.is_installed", return_value=False)
    def test_valid_parallel_run(self, mock_installed, mock_popen, tmp_path):
        script = tmp_path / "test.json"
        script.write_text("{}", encoding="utf-8")

        mock_proc = MagicMock()
        mock_proc.poll.return_value = None
        mock_proc.returncode = None
        # Simulate process completing on second poll
        def side_effect():
            mock_proc.returncode = 0
            return 0
        mock_proc.poll.side_effect = [None, side_effect()]
        mock_popen.return_value = mock_proc

        step = {
            "parallel_run": {
                "runners": ["api-runner"],
                "scripts": [str(script)],
            }
        }
        result = parallel_run(step)
        assert result is True

    def test_unknown_runner_skips(self, tmp_path):
        script = tmp_path / "test.json"
        script.write_text("{}", encoding="utf-8")

        step = {
            "parallel_run": {
                "runners": ["unknown-runner"],
                "scripts": [str(script)],
            }
        }
        result = parallel_run(step)
        assert result is True  # Returns True after skipping unknown runner



"""Tests for test_pioneer.executor.run.utils"""
from unittest.mock import patch, MagicMock

from test_pioneer.executor.run.utils import select_with_runner


# Common mock modules for runner imports
_MOCK_RUNNER_MODULES = {
    "je_web_runner": MagicMock(execute_action=MagicMock()),
    "je_api_testka": MagicMock(execute_action=MagicMock()),
    "je_load_density": MagicMock(execute_action=MagicMock()),
}


class TestSelectWithRunner:
    def test_missing_with_tag(self):
        result = select_with_runner({"run": "test.json"}, enable_logging=False)
        assert result == (False, None)

    def test_non_string_with_tag(self):
        result = select_with_runner({"with": 123}, enable_logging=False)
        assert result == (False, None)

    @patch.dict("sys.modules", _MOCK_RUNNER_MODULES)
    @patch("test_pioneer.executor.run.utils.is_installed", return_value=False)
    def test_gui_runner_not_installed(self, mock_installed):
        step = {"with": "gui-runner", "run": "test.json"}
        ok, runner = select_with_runner(step, enable_logging=False, mode="run")
        assert ok is False
        assert runner is None

    @patch.dict("sys.modules", _MOCK_RUNNER_MODULES)
    @patch("test_pioneer.executor.run.utils.is_installed", return_value=False)
    def test_invalid_runner_tag(self, mock_installed):
        step = {"with": "invalid-runner", "run": "test.json"}
        ok, runner = select_with_runner(step, enable_logging=False, mode="run")
        assert ok is False
        assert runner is None

    @patch.dict("sys.modules", _MOCK_RUNNER_MODULES)
    @patch("test_pioneer.executor.run.utils.is_installed", return_value=False)
    def test_api_runner_selection(self, mock_installed):
        step = {"with": "api-runner", "run": "test.json"}
        ok, runner = select_with_runner(step, enable_logging=False, mode="run")
        assert ok is True
        assert runner is not None

    @patch.dict("sys.modules", _MOCK_RUNNER_MODULES)
    @patch("test_pioneer.executor.run.utils.is_installed", return_value=False)
    def test_web_runner_selection(self, mock_installed):
        step = {"with": "web-runner", "run": "test.json"}
        ok, runner = select_with_runner(step, enable_logging=False, mode="run")
        assert ok is True
        assert runner is not None

    @patch.dict("sys.modules", _MOCK_RUNNER_MODULES)
    @patch("test_pioneer.executor.run.utils.is_installed", return_value=False)
    def test_load_runner_selection(self, mock_installed):
        step = {"with": "load-runner", "run": "test.json"}
        ok, runner = select_with_runner(step, enable_logging=False, mode="run")
        assert ok is True
        assert runner is not None

    @patch.dict("sys.modules", _MOCK_RUNNER_MODULES)
    @patch("test_pioneer.executor.run.utils.is_installed", return_value=False)
    def test_invalid_mode_returns_false(self, mock_installed):
        step = {"with": "api-runner", "run": "test.json"}
        ok, runner = select_with_runner(step, enable_logging=False, mode="invalid")
        assert ok is False
        assert runner is None

    @patch.dict("sys.modules", _MOCK_RUNNER_MODULES)
    @patch("test_pioneer.executor.run.utils.is_installed", return_value=False)
    def test_gui_runner_not_installed_run_folder_mode(self, mock_installed):
        step = {"with": "gui-runner", "run_folder": "./test"}
        ok, runner = select_with_runner(step, enable_logging=False, mode="run_folder")
        assert ok is False
        assert runner is None



"""Tests for test_pioneer.utils modules"""
from test_pioneer.utils.package.check import is_installed
from test_pioneer.utils.exception.exceptions import (
    ExecutorException, YamlException, WrongInputException, ProjectException,
)


class TestIsInstalled:
    def test_installed_package(self):
        assert is_installed("os") is True

    def test_not_installed_package(self):
        assert is_installed("nonexistent_package_xyz_12345") is False

    def test_stdlib_package(self):
        assert is_installed("json") is True


class TestExceptions:
    def test_executor_exception(self):
        exc = ExecutorException("test")
        assert str(exc) == "test"
        assert isinstance(exc, Exception)

    def test_yaml_exception(self):
        exc = YamlException("test")
        assert str(exc) == "test"
        assert isinstance(exc, Exception)

    def test_wrong_input_exception(self):
        exc = WrongInputException("test")
        assert str(exc) == "test"
        assert isinstance(exc, Exception)

    def test_project_exception(self):
        exc = ProjectException("test")
        assert str(exc) == "test"
        assert isinstance(exc, Exception)



"""Tests for test_pioneer.process modules"""
from unittest.mock import patch, MagicMock

from test_pioneer.process.execute_process import ExecuteProcess
from test_pioneer.process.process_manager import ProcessManager


class TestExecuteProcess:
    def test_init_defaults(self):
        ep = ExecuteProcess()
        assert ep.process is None
        import subprocess
        assert ep.redirect_stdout == subprocess.PIPE
        assert ep.redirect_stderr == subprocess.PIPE

    @patch("test_pioneer.process.execute_process.subprocess.Popen")
    def test_start_process_pipe(self, mock_popen):
        ep = ExecuteProcess()
        ep.start_process("echo hello")
        mock_popen.assert_called_once()
        assert ep.process is not None

    @patch("test_pioneer.process.execute_process.subprocess.Popen")
    def test_start_process_with_file_redirect(self, mock_popen, tmp_path):
        stdout_path = str(tmp_path / "out.log")
        stderr_path = str(tmp_path / "err.log")
        ep = ExecuteProcess(redirect_stdout=stdout_path, redirect_stderr=stderr_path)
        ep.start_process("echo hello")
        mock_popen.assert_called_once()
        # Verify file handles were opened
        assert ep._stdout_file is not None
        assert ep._stderr_file is not None
        ep._stdout_file.close()
        ep._stderr_file.close()

    def test_exit_program_no_process(self):
        ep = ExecuteProcess()
        ep.exit_program()  # Should not raise

    @patch("test_pioneer.process.execute_process.psutil.Process")
    def test_exit_program_kills_tree(self, mock_psutil):
        ep = ExecuteProcess()
        ep.process = MagicMock()
        ep.process.pid = 12345

        mock_child = MagicMock()
        mock_psutil.return_value.children.return_value = [mock_child]

        ep.exit_program()
        mock_child.kill.assert_called_once()
        mock_psutil.return_value.kill.assert_called_once()


class TestProcessManager:
    def test_close_process_existing(self):
        pm = ProcessManager()
        mock_ep = MagicMock()
        pm.process_dict["test"] = mock_ep
        pm.name_set.add("test")

        pm.close_process("test")
        mock_ep.exit_program.assert_called_once()
        assert "test" not in pm.process_dict
        assert "test" not in pm.name_set

    def test_close_process_nonexistent(self):
        pm = ProcessManager()
        pm.close_process("nonexistent")  # Should not raise



"""Tests for test_pioneer.executor.time.wait"""
from unittest.mock import patch

from test_pioneer.executor.time.wait import blocked_wait


class TestBlockedWait:
    @patch("test_pioneer.executor.time.wait.time.sleep")
    def test_valid_wait(self, mock_sleep):
        result = blocked_wait({"wait": 3})
        assert result is True
        mock_sleep.assert_called_once_with(3)

    def test_non_int_returns_false(self):
        result = blocked_wait({"wait": "not_int"})
        assert result is False

    def test_missing_wait_returns_false(self):
        result = blocked_wait({})
        assert result is False

    @patch("test_pioneer.executor.time.wait.time.sleep")
    def test_zero_wait(self, mock_sleep):
        result = blocked_wait({"wait": 0})
        assert result is True
        mock_sleep.assert_called_once_with(0)



"""Tests for test_pioneer.executor.program.external_program"""
from unittest.mock import patch, MagicMock

from test_pioneer.executor.program.external_program import open_program, close_program
from test_pioneer.process.process_manager import process_manager_instance


class TestOpenProgram:
    @patch("test_pioneer.executor.program.external_program.ExecuteProcess")
    def test_open_program_basic(self, mock_ep_class):
        mock_ep = MagicMock()
        mock_ep_class.return_value = mock_ep

        step = {"open_program": "notepad.exe"}
        result = open_program(step, name="test_notepad")
        assert result is True
        mock_ep.start_process.assert_called_once_with("notepad.exe")
        assert "test_notepad" in process_manager_instance.process_dict

    @patch("test_pioneer.executor.program.external_program.ExecuteProcess")
    def test_open_program_with_redirects(self, mock_ep_class):
        mock_ep = MagicMock()
        mock_ep_class.return_value = mock_ep

        step = {
            "open_program": "notepad.exe",
            "redirect_stdout": "stdout.log",
            "redirect_stderr": "stderr.log",
        }
        result = open_program(step, name="test_notepad2")
        assert result is True
        assert mock_ep.redirect_stdout == "stdout.log"
        assert mock_ep.redirect_stderr == "stderr.log"

    def test_non_string_program_returns_false(self):
        result = open_program({"open_program": 123}, name="test")
        assert result is False

    def test_non_string_stdout_returns_false(self):
        step = {"open_program": "notepad.exe", "redirect_stdout": 123}
        result = open_program(step, name="test")
        assert result is False

    def test_non_string_stderr_returns_false(self):
        step = {"open_program": "notepad.exe", "redirect_stderr": 123}
        result = open_program(step, name="test")
        assert result is False


class TestCloseProgram:
    def test_close_program_valid(self):
        mock_ep = MagicMock()
        process_manager_instance.process_dict["my_prog"] = mock_ep
        process_manager_instance.name_set.add("my_prog")

        result = close_program({"close_program": "my_prog"})
        assert result is True
        mock_ep.exit_program.assert_called_once()

    def test_close_program_not_found(self):
        result = close_program({"close_program": "nonexistent"})
        assert result is True  # close_program always returns True

    def test_non_string_returns_false(self):
        result = close_program({"close_program": 123})
        assert result is False



"""Tests for test_pioneer.executor.browser.url"""
from unittest.mock import patch

from test_pioneer.executor.browser.url import open_url


class TestOpenUrl:
    def test_valid_url_default_method(self):
        with patch("test_pioneer.executor.browser.url.webbrowser") as mock_wb:
            result = open_url({"open_url": "https://example.com"})
            assert result is True
            mock_wb.open.assert_called_once_with(url="https://example.com")

    def test_valid_url_open_new_tab(self):
        with patch("test_pioneer.executor.browser.url.webbrowser") as mock_wb:
            step = {"open_url": "https://example.com", "url_open_method": "open_new_tab"}
            result = open_url(step)
            assert result is True
            mock_wb.open_new_tab.assert_called_once_with(url="https://example.com")

    def test_valid_url_open_new(self):
        with patch("test_pioneer.executor.browser.url.webbrowser") as mock_wb:
            step = {"open_url": "https://example.com", "url_open_method": "open_new"}
            result = open_url(step)
            assert result is True
            mock_wb.open_new.assert_called_once_with(url="https://example.com")

    def test_non_string_url_returns_false(self):
        result = open_url({"open_url": 123})
        assert result is False

    def test_missing_url_returns_false(self):
        result = open_url({})
        assert result is False

    def test_invalid_method_returns_false(self):
        with patch("test_pioneer.executor.browser.url.webbrowser"):
            step = {"open_url": "https://example.com", "url_open_method": "invalid"}
            result = open_url(step)
            assert result is False

    def test_exception_returns_false(self):
        with patch("test_pioneer.executor.browser.url.webbrowser") as mock_wb:
            mock_wb.open.side_effect = Exception("browser error")
            result = open_url({"open_url": "https://example.com"})
            assert result is False



"""Tests for test_pioneer.executor.file.file_processing"""
import sys
from unittest.mock import patch, MagicMock

# Mock automation_file before importing the module under test
mock_automation_file = MagicMock()
sys.modules["automation_file"] = mock_automation_file

from test_pioneer.executor.file.file_processing import download_single_file, unzip_zipfile


class TestDownloadSingleFile:
    def setup_method(self):
        mock_automation_file.reset_mock()

    def test_valid_download(self):
        step = {"download_file": "http://example.com/f.zip", "file_path": "f.zip"}
        result = download_single_file(step)
        assert result is True
        mock_automation_file.download_file.assert_called_once_with(
            file_url="http://example.com/f.zip", file_name="f.zip"
        )

    def test_missing_url_returns_false(self):
        result = download_single_file({"file_path": "f.zip"})
        assert result is False

    def test_missing_file_path_returns_false(self):
        result = download_single_file({"download_file": "http://example.com/f.zip"})
        assert result is False

    def test_non_string_url_returns_false(self):
        result = download_single_file({"download_file": 123, "file_path": "f.zip"})
        assert result is False

    def test_non_string_path_returns_false(self):
        result = download_single_file({"download_file": "http://example.com/f.zip", "file_path": 123})
        assert result is False


class TestUnzipZipfile:
    def setup_method(self):
        mock_automation_file.reset_mock()

    def test_valid_unzip(self):
        step = {"zip_file_path": "test.zip"}
        result = unzip_zipfile(step)
        assert result is True
        mock_automation_file.unzip_all.assert_called_once_with(zip_file_path="test.zip")

    def test_unzip_with_password_and_extract_path(self):
        step = {"zip_file_path": "test.zip", "password": "secret", "extract_path": "/tmp"}
        result = unzip_zipfile(step)
        assert result is True
        mock_automation_file.unzip_all.assert_called_once_with(
            zip_file_path="test.zip", password="secret", extract_path="/tmp"
        )

    def test_missing_zip_path_returns_false(self):
        result = unzip_zipfile({})
        assert result is False

    def test_non_string_zip_path_returns_false(self):
        result = unzip_zipfile({"zip_file_path": 123})
        assert result is False



"""Tests for test_pioneer.logging.loggin_instance and test_recorder.logger"""
import logging
from unittest.mock import MagicMock

from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger, TestPioneerHandler
from test_pioneer.executor.test_recorder.logger import set_logger


class TestStepLogCheck:
    def test_logs_info_when_enabled(self):
        mock_logger = MagicMock(spec=logging.Logger)
        step_log_check(enable_logging=True, logger=mock_logger, level="info", message="test msg")
        mock_logger.info.assert_called_once_with("test msg")

    def test_logs_error_when_enabled(self):
        mock_logger = MagicMock(spec=logging.Logger)
        step_log_check(enable_logging=True, logger=mock_logger, level="error", message="error msg")
        mock_logger.error.assert_called_once_with("error msg")

    def test_no_log_when_disabled(self):
        mock_logger = MagicMock(spec=logging.Logger)
        step_log_check(enable_logging=False, logger=mock_logger, level="info", message="test msg")
        mock_logger.info.assert_not_called()

    def test_no_log_when_no_message(self):
        mock_logger = MagicMock(spec=logging.Logger)
        step_log_check(enable_logging=True, logger=mock_logger, level="info", message=None)
        mock_logger.info.assert_not_called()

    def test_invalid_level_does_nothing(self):
        mock_logger = MagicMock(spec=logging.Logger)
        step_log_check(enable_logging=True, logger=mock_logger, level="warning", message="msg")
        mock_logger.info.assert_not_called()
        mock_logger.error.assert_not_called()


class TestTestPioneerHandler:
    def test_handler_creation(self, tmp_path):
        log_file = str(tmp_path / "test.log")
        handler = TestPioneerHandler(filename=log_file)
        assert handler.maxBytes == 1073741824
        assert handler.backupCount == 0
        handler.close()

    def test_handler_emit(self, tmp_path):
        log_file = str(tmp_path / "test.log")
        handler = TestPioneerHandler(filename=log_file)
        record = logging.LogRecord(
            name="test", level=logging.INFO, pathname="", lineno=0,
            msg="test message", args=None, exc_info=None
        )
        handler.emit(record)
        handler.close()
        with open(log_file, "r") as f:
            content = f.read()
        assert "test message" in content


class TestSetLogger:
    def test_set_logger_with_pioneer_log(self, tmp_path):
        log_file = str(tmp_path / "test.log")
        result = set_logger({"pioneer_log": log_file})
        assert result is True
        # Clean up handler
        for handler in test_pioneer_logger.handlers[:]:
            if isinstance(handler, TestPioneerHandler):
                handler.close()
                test_pioneer_logger.removeHandler(handler)

    def test_set_logger_without_pioneer_log(self):
        result = set_logger({})
        assert result is False

    def test_set_logger_with_none_value(self):
        result = set_logger({"pioneer_log": None})
        assert result is False



"""Tests for test_pioneer.project.create_template_structure"""
from pathlib import Path

from test_pioneer.project.create_template_structure import create_dir, create_template, create_template_dir


class TestCreateDir:
    def test_create_new_directory(self, tmp_path):
        new_dir = tmp_path / "test_dir"
        create_dir(str(new_dir))
        assert new_dir.is_dir()

    def test_create_nested_directory(self, tmp_path):
        new_dir = tmp_path / "a" / "b" / "c"
        create_dir(str(new_dir))
        assert new_dir.is_dir()

    def test_create_existing_directory(self, tmp_path):
        existing = tmp_path / "existing"
        existing.mkdir()
        create_dir(str(existing))  # Should not raise
        assert existing.is_dir()


class TestCreateTemplate:
    def test_create_template_file(self, tmp_path):
        parent = ".TestPioneer"
        template_dir = tmp_path / parent
        template_dir.mkdir()

        create_template(parent, str(tmp_path))

        template_file = template_dir / f"{parent}.yml"
        assert template_file.is_file()
        content = template_file.read_text(encoding="utf-8")
        assert "jobs:" in content

    def test_create_template_dir_not_exists(self, tmp_path):
        # Should do nothing if directory doesn't exist
        create_template("nonexistent", str(tmp_path))


class TestCreateTemplateDir:
    def test_full_workflow(self, tmp_path):
        create_template_dir(project_path=str(tmp_path), parent_name=".TestProject")

        project_dir = tmp_path / ".TestProject"
        assert project_dir.is_dir()

        template_file = project_dir / ".TestProject.yml"
        assert template_file.is_file()



"""Tests for test_pioneer.executor.run.process_manager"""
from unittest.mock import MagicMock

from test_pioneer.executor.run.process_manager import ProcessManager, process_manager


class TestProcessManagerRun:
    def test_add_process(self):
        pm = ProcessManager()
        mock_proc = MagicMock()
        pm.add_process(mock_proc)
        assert mock_proc in pm.process_list

    def test_remove_process(self):
        pm = ProcessManager()
        mock_proc = MagicMock()
        pm.add_process(mock_proc)
        pm.remove_process(mock_proc)
        assert mock_proc not in pm.process_list

    def test_remove_nonexistent_process(self):
        pm = ProcessManager()
        mock_proc = MagicMock()
        pm.remove_process(mock_proc)  # Should not raise

    def test_cleanup_finished(self):
        pm = ProcessManager()
        finished = MagicMock()
        finished.returncode = 0
        running = MagicMock()
        running.returncode = None

        pm.process_list = [finished, running]
        pm.cleanup_finished()
        assert finished not in pm.process_list
        assert running in pm.process_list

    def test_terminate_all(self):
        pm = ProcessManager()
        proc1 = MagicMock()
        proc2 = MagicMock()
        pm.process_list = [proc1, proc2]

        pm.terminate_all()
        proc1.terminate.assert_called_once()
        proc2.terminate.assert_called_once()
        assert len(pm.process_list) == 0

    def test_global_instance_exists(self):
        assert process_manager is not None
        assert isinstance(process_manager, ProcessManager)



"""Tests for test_pioneer.executor.pioneer_executor"""
import pytest
from unittest.mock import patch, MagicMock

from test_pioneer.executor.pioneer_executor import execute_yaml, _load_yaml, _validate_steps
from test_pioneer.utils.exception.exceptions import WrongInputException, YamlException


class TestLoadYaml:
    def test_load_from_string(self):
        yaml_str = "jobs:\n  steps:\n    - name: test"
        result = _load_yaml(yaml_str, "String")
        assert isinstance(result, dict)
        assert "jobs" in result

    def test_load_from_file(self, tmp_path):
        yaml_file = tmp_path / "test.yml"
        yaml_file.write_text("jobs:\n  steps:\n    - name: test", encoding="utf-8")
        result = _load_yaml(str(yaml_file), "File")
        assert isinstance(result, dict)

    def test_load_invalid_type_raises(self):
        with pytest.raises(WrongInputException):
            _load_yaml("some_data", "InvalidType")

    def test_load_non_dict_raises(self):
        with pytest.raises(YamlException, match="Not a dict"):
            _load_yaml("- item1\n- item2", "String")


class TestValidateSteps:
    def test_valid_steps(self):
        steps = [{"name": "step1"}, {"name": "step2"}]
        assert _validate_steps(steps, enable_logging=False) is True

    def test_missing_name(self):
        steps = [{"run": "test.json"}]
        assert _validate_steps(steps, enable_logging=False) is False

    def test_duplicate_name(self):
        steps = [{"name": "dup"}, {"name": "dup"}]
        assert _validate_steps(steps, enable_logging=False) is False


class TestExecuteYaml:
    def test_no_jobs_tag_raises(self):
        with pytest.raises(YamlException, match="No jobs tag"):
            execute_yaml("key: value", yaml_type="String")

    def test_jobs_not_dict_raises(self):
        with pytest.raises(YamlException, match="jobs not a dict"):
            execute_yaml("jobs: not_a_dict", yaml_type="String")

    def test_empty_steps_raises(self):
        with pytest.raises(YamlException, match="Steps tag is empty"):
            execute_yaml("jobs:\n  steps:", yaml_type="String")

    @patch("test_pioneer.executor.pioneer_executor.blocked_wait", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_wait_step(self, mock_installed, mock_logger, mock_wait):
        yaml_str = "jobs:\n  steps:\n    - name: wait_test\n      wait: 5"
        execute_yaml(yaml_str, yaml_type="String")
        mock_wait.assert_called_once()
        step_arg = mock_wait.call_args[1]["step"]
        assert step_arg["wait"] == 5

    @patch("test_pioneer.executor.pioneer_executor.open_url", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_open_url_step(self, mock_installed, mock_logger, mock_open_url):
        yaml_str = "jobs:\n  steps:\n    - name: url_test\n      open_url: https://example.com"
        execute_yaml(yaml_str, yaml_type="String")
        mock_open_url.assert_called_once()

    @patch("test_pioneer.executor.pioneer_executor.run", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_step_failure_stops_execution(self, mock_installed, mock_logger, mock_run):
        yaml_str = (
            "jobs:\n  steps:\n"
            "    - name: step1\n      run: test.json\n      with: api-runner\n"
            "    - name: step2\n      run: test2.json\n      with: api-runner"
        )
        execute_yaml(yaml_str, yaml_type="String")
        # Only the first step should be called since it returned False
        assert mock_run.call_count == 1

    @patch("test_pioneer.executor.pioneer_executor.download_single_file", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_download_file_step(self, mock_installed, mock_logger, mock_download):
        yaml_str = (
            "jobs:\n  steps:\n"
            "    - name: dl_test\n      download_file: http://example.com/f.zip\n      file_path: f.zip"
        )
        execute_yaml(yaml_str, yaml_type="String")
        mock_download.assert_called_once()

    @patch("test_pioneer.executor.pioneer_executor.open_program", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_open_program_step(self, mock_installed, mock_logger, mock_open):
        yaml_str = "jobs:\n  steps:\n    - name: prog_test\n      open_program: notepad.exe"
        execute_yaml(yaml_str, yaml_type="String")
        mock_open.assert_called_once()
        assert mock_open.call_args[1]["name"] == "prog_test"

    @patch("test_pioneer.executor.pioneer_executor.close_program", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_close_program_step(self, mock_installed, mock_logger, mock_close):
        yaml_str = "jobs:\n  steps:\n    - name: close_test\n      close_program: notepad.exe"
        execute_yaml(yaml_str, yaml_type="String")
        mock_close.assert_called_once()

    @patch("test_pioneer.executor.pioneer_executor.unzip_zipfile", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_unzip_step(self, mock_installed, mock_logger, mock_unzip):
        yaml_str = "jobs:\n  steps:\n    - name: zip_test\n      unzip_zipfile: test.zip"
        execute_yaml(yaml_str, yaml_type="String")
        mock_unzip.assert_called_once()

    @patch("test_pioneer.executor.pioneer_executor.parallel_run", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_parallel_run_step(self, mock_installed, mock_logger, mock_parallel):
        yaml_str = (
            "jobs:\n  steps:\n    - name: par_test\n"
            "      parallel_run:\n        runners: [api-runner]\n        scripts: [test.json]"
        )
        execute_yaml(yaml_str, yaml_type="String")
        mock_parallel.assert_called_once()

    @patch("test_pioneer.executor.pioneer_executor.run_folder", return_value=True)
    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_run_folder_step(self, mock_installed, mock_logger, mock_run_folder):
        yaml_str = "jobs:\n  steps:\n    - name: folder_test\n      run_folder: ./test\n      with: api-runner"
        execute_yaml(yaml_str, yaml_type="String")
        mock_run_folder.assert_called_once()

    @patch("test_pioneer.executor.pioneer_executor.set_logger", return_value=False)
    @patch("test_pioneer.executor.pioneer_executor.is_installed", return_value=False)
    def test_duplicate_step_names_returns_early(self, mock_installed, mock_logger):
        yaml_str = (
            "jobs:\n  steps:\n"
            "    - name: dup\n      wait: 1\n"
            "    - name: dup\n      wait: 2"
        )
        # Should return without error (just returns None early)
        execute_yaml(yaml_str, yaml_type="String")



import argparse

from test_pioneer import execute_yaml
from test_pioneer.utils.exception.exceptions import ExecutorException

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="TestPioneer - Automation test framework for CI/CD"
    )

    parser.add_argument(
        "-e", "--execute_yaml",
        type=str, help="choose yaml file to execute"
    )
    args = parser.parse_args()
    args = vars(args)
    if args.get("execute_yaml"):
        execute_yaml(args.get("execute_yaml"))
    else:
        raise ExecutorException(
            "execute_yaml have no argument, usage: python -m test_pioneer -e <filepath>")



import time
from pathlib import Path

import yaml

from test_pioneer.executor.browser.url import open_url
from test_pioneer.executor.file.file_processing import download_single_file, unzip_zipfile
from test_pioneer.executor.program.external_program import open_program, close_program
from test_pioneer.executor.run.executor_run import run
from test_pioneer.executor.run.executor_run_folder import run_folder
from test_pioneer.executor.run.parallel_run import parallel_run
from test_pioneer.executor.test_recorder.logger import set_logger
from test_pioneer.executor.time.wait import blocked_wait
from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger
from test_pioneer.process.process_manager import process_manager_instance
from test_pioneer.utils.exception.exceptions import WrongInputException, YamlException
from test_pioneer.utils.package.check import is_installed

# Step type to handler mapping
# 步驟類型對應的處理函式
_STEP_HANDLERS = {
    "run": lambda step, name, enable_logging: run(step=step, enable_logging=enable_logging),
    "run_folder": lambda step, name, enable_logging: run_folder(step=step, enable_logging=enable_logging, mode="run_folder"),
    "open_url": lambda step, name, enable_logging: open_url(step=step, enable_logging=enable_logging),
    "download_file": lambda step, name, enable_logging: download_single_file(step=step, enable_logging=enable_logging),
    "wait": lambda step, name, enable_logging: blocked_wait(step=step, enable_logging=enable_logging),
    "open_program": lambda step, name, enable_logging: open_program(step=step, name=name, enable_logging=enable_logging),
    "close_program": lambda step, name, enable_logging: close_program(step=step, enable_logging=enable_logging),
    "unzip_zipfile": lambda step, name, enable_logging: unzip_zipfile(step=step, enable_logging=enable_logging),
    "parallel_run": lambda step, name, enable_logging: parallel_run(step=step, enable_logging=enable_logging),
}


def _stop_recorder(recording: bool, recorder) -> None:
    """Stop the recording thread if active."""
    if recording and recorder is not None:
        recorder.set_recording_flag(False)
        while recorder.is_alive():
            time.sleep(0.1)


def _load_yaml(stream: str, yaml_type: str) -> dict:
    """Load and validate YAML data from file or string."""
    if yaml_type == "File":
        yaml_data = yaml.safe_load(Path(stream).read_text(encoding="utf-8"))
    elif yaml_type == "String":
        yaml_data = yaml.safe_load(stream=stream)
    else:
        raise WrongInputException("Wrong input: " + repr(stream))

    if not isinstance(yaml_data, dict):
        raise YamlException(f"Not a dict: {yaml_data}")
    return yaml_data


def _validate_steps(steps: list, enable_logging: bool) -> bool:
    """Validate step names for duplicates. Returns True if valid."""
    for step in steps:
        if step.get("name") is None:
            step_log_check(
                enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
                message="Step need name tag")
            return False
        name = step.get("name")
        if name in process_manager_instance.name_set:
            step_log_check(
                enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
                message=f"job name duplicated: {name}")
            return False
        process_manager_instance.name_set.add(name)
    return True


def execute_yaml(stream: str, yaml_type: str = "File"):
    recording = False
    recorder = None

    yaml_data = _load_yaml(stream, yaml_type)

    # Pre-check save log or not
    enable_logging = set_logger(yaml_data=yaml_data)
    if is_installed(package_name="je_auto_control"):
        from test_pioneer.executor.test_recorder.video_recoder import set_recorder
        recording, recorder = set_recorder(yaml_data=yaml_data)

    try:
        # Pre-check jobs
        if "jobs" not in yaml_data:
            raise YamlException("No jobs tag")
        if not isinstance(yaml_data.get("jobs"), dict):
            raise YamlException("jobs not a dict")

        # Pre-check steps
        steps = yaml_data["jobs"].get("steps")
        if not steps:
            raise YamlException("Steps tag is empty")

        if not _validate_steps(steps, enable_logging):
            return

        # Execute step actions
        for step in steps:
            name = step.get("name")
            for step_type, handler in _STEP_HANDLERS.items():
                if step_type in step:
                    if not handler(step, name, enable_logging):
                        return
                    break

    except Exception as error:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"Error: {repr(error)}")
        raise error
    finally:
        if is_installed(package_name="je_auto_control"):
            _stop_recorder(recording, recorder)
